@@ -60,23 +60,28 @@ class Action:
         *,
         picUrl: str = "",
         picBase64Buf: str = "",
-        fileMd5: str = "",
+        picMd5s: Union[str, List[str]] = None,
         content: str = "",
         flashPic=False,
     ):
         """发送好友图片消息"""
-        assert any([picUrl, picBase64Buf, fileMd5]), "缺少参数"
+        assert any([picUrl, picBase64Buf, picMd5s]), "缺少参数"
+        if isinstance(picMd5s, str):
+            picMd5s = [picMd5s]
+        picMd5s = [  # type: ignore
+            {"FileId": 1, "PicMd5": picmd5, "PicSize": 1} for picmd5 in picMd5s or []
+        ]
         return self.post(
-            "SendMsg",
+            "SendMsgV2",
             {
-                "toUser": user,
-                "sendToType": 1,
-                "sendMsgType": "PicMsg",
-                "content": content,
-                "picUrl": picUrl,
-                "picBase64Buf": picBase64Buf,
-                "fileMd5": fileMd5,
-                "flashPic": flashPic,
+                "ToUserUid": user,
+                "SendToType": 1,
+                "SendMsgType": "PicMsg",
+                "Content": content,
+                "PicUrl": picUrl,
+                "PicBase64Buf": picBase64Buf,
+                "PicMd5s": picMd5s,
+                "FlashPic": flashPic,
             },
         )
 
@@ -143,41 +148,53 @@ class Action:
         content: str = "",
         picUrl: str = "",
         picBase64Buf: str = "",
-        fileMd5: str = "",
-        picMd5s: Union[str, List[str]] = "",
+        picMd5s: Union[str, List[str]] = None,
         flashPic=False,
         atUser: Union[int, List[int]] = 0,
     ) -> dict:
         """发送群组图片消息"""
-        assert any([picUrl, picBase64Buf, fileMd5, picMd5s]), "缺少参数"
+        assert any([picUrl, picBase64Buf, picMd5s, picMd5s]), "缺少参数"
         if atUser != 0:
             content = macro.atUser(atUser) + "\n" + content
-        if picMd5s:
-            if isinstance(picMd5s, str):
-                picMd5s = [picMd5s]
-            return self.post(
-                "SendMsgV2",
-                {
-                    "ToUserUid": group,
-                    "SendToType": 2,
-                    "SendMsgType": "PicMsg",
-                    "PicMd5s": picMd5s,
-                },
-            )
+        if isinstance(picMd5s, str):
+            picMd5s = [picMd5s]
+        picMd5s = [  # type: ignore
+            {"FileId": 1, "PicMd5": picmd5, "PicSize": 1} for picmd5 in picMd5s or []
+        ]
         return self.post(
-            "SendMsg",
+            "SendMsgV2",
             {
-                "toUser": group,
-                "sendToType": 2,
-                "sendMsgType": "PicMsg",
-                "content": content,
-                "picUrl": picUrl,
-                "picBase64Buf": picBase64Buf,
-                "fileMd5": fileMd5,
-                "flashPic": flashPic,
+                "ToUserUid": group,
+                "SendToType": 2,
+                "SendMsgType": "PicMsg",
+                "Content": content,
+                "PicUrl": picUrl,
+                "PicBase64Buf": picBase64Buf,
+                "PicMd5s": picMd5s,
+                "FlashPic": flashPic,
             },
         )
 
+    def sendGroupMultiPic(
+        self,
+        group,
+        *items: str,
+        text: str = "",
+        atUser: Union[int, List[int]] = 0,
+    ):
+        """发送群多图
+        items 支持填写图片http地址和base64，会自动判断类型
+        """
+        md5s = []
+        for item in items:
+            if item.startswith("http"):
+                info = self.getGroupPicInfo(url=item)
+            else:
+                info = self.getGroupPicInfo(base64=item)
+            md5s.append(info["PicInfo"]["PicMd5"])
+            time.sleep(0.5)
+        return self.sendGroupPic(group, content=text, atUser=atUser, picMd5s=md5s)
+
     def sendGroupVoice(
         self, group: int, *, voiceUrl: str = "", voiceBase64Buf: str = ""
     ) -> dict:
@@ -281,21 +298,26 @@ class Action:
         *,
         picUrl: str = "",
         picBase64Buf: str = "",
-        fileMd5: str = "",
+        picMd5s: Union[str, List[str]] = None,
     ) -> dict:
         """发送私聊图片消息"""
-        assert any([picUrl, picBase64Buf, fileMd5]), "缺少参数"
+        assert any([picUrl, picBase64Buf, picMd5s]), "缺少参数"
+        if isinstance(picMd5s, str):
+            picMd5s = [picMd5s]
+        picMd5s = [  # type: ignore
+            {"FileId": 1, "PicMd5": picmd5, "PicSize": 1} for picmd5 in picMd5s or []
+        ]
         return self.post(
-            "SendMsg",
+            "SendMsgV2",
             {
-                "toUser": user,
-                "sendToType": 3,
-                "sendMsgType": "PicMsg",
-                "content": content,
-                "groupid": group,
-                "picUrl": picUrl,
-                "picBase64Buf": picBase64Buf,
-                "fileMd5": fileMd5,
+                "ToUserUid": user,
+                "GroupID": group,
+                "SendToType": 3,
+                "SendMsgType": "PicMsg",
+                "Content": content,
+                "PicUrl": picUrl,
+                "PicBase64Buf": picBase64Buf,
+                "PicMd5s": picMd5s,
             },
         )
 
@@ -667,12 +689,11 @@ class Action:
             },
         )
 
-    def likeUser(self, user: int, cmd=0) -> dict:
+    def likeUser(self, user: int) -> dict:
         """给某人点赞
         :param user: 用户QQ号
-        :param cmd: 发送包选项, 0 或 1; 0表示使用``QQZan``; 1表示使用``OidbSvc.0x7e5_4``, 默认为0
         """
-        return self.post("QQZan" if cmd == 0 else "OidbSvc.0x7e5_4", {"UserID": user})
+        return self.post("QQZan", {"UserID": user})
 
     def toggleGroupAdmin(self, user: int, group: int, flag=1) -> dict:
         """设置和取消群管理员
@@ -685,20 +706,20 @@ class Action:
             {"GroupID": group, "UserID": user, "Flag": 0 if flag == 0 else 1},
         )
 
-    def revokeGroupMsg(
-        self, group: int, msgSeq: int, msgRandom: int, cmd: int = 0
-    ) -> dict:
+    def revokeGroupMsg(self, group: int, msgSeq: int, msgRandom: int) -> dict:
         """撤回群消息
         :param group: 群号
         :param msgSeq: 消息msgSeq
         :param msgRandom: 消息msgRandom
-        :param cmd: 0 或 1, 0表示使用: RevokeMsg , 1表示使用: PbMessageSvc.PbMsgWithDraw
         """
         return self.post(
-            "RevokeMsg" if cmd == 0 else "PbMessageSvc.PbMsgWithDraw",
-            {"GroupID": group, "MsgSeq": msgSeq, "MsgRandom": msgRandom},
+            "RevokeMsg", {"GroupID": group, "MsgSeq": msgSeq, "MsgRandom": msgRandom}
         )
 
+    def revoke(self, ctx: GroupMsg):
+        """撤回群消息"""
+        return self.revokeGroupMsg(ctx.FromGroupId, ctx.MsgSeq, ctx.MsgRandom)
+
     def inviteUserJoinGroup(self, group: int, user: int) -> dict:
         """拉人入群
         :param group: 哪个群?
@@ -748,6 +769,22 @@ class Action:
         """
         return self.post("LogOut", {"Flag": flag})
 
+    def getGroupPicInfo(self, url: str = "", base64: str = ""):
+        """上传群图片获取图片信息
+        :param url: 图片链接
+        :param base64: 图片base64
+        """
+        assert any([url, base64]), "缺少参数"
+        return self.post(
+            "", {"PicUrl": url, "PicBase64Buf": base64}, path="/v1/GetGroupPicInfo"
+        )
+
+    def updateAvatar(self, url: str):
+        """上传头像
+        :param url: 图片链接
+        """
+        return self.post("", {"HDIMGUrl": url}, path="/v1/SelfHDIMG")
+
     ############################################################################
     def baseRequest(
         self,
