@@ -64,23 +64,28 @@ class AsyncAction:
         *,
         picUrl: str = "",
         picBase64Buf: str = "",
-        fileMd5: str = "",
+        picMd5s: Union[str, List[str]] = None,
         content: str = "",
         flashPic=False,
     ):
         """发送好友图片消息"""
-        assert any([picUrl, picBase64Buf, fileMd5]), "缺少参数"
-        return await self.post(
-            "SendMsg",
+        assert any([picUrl, picBase64Buf, picMd5s]), "缺少参数"
+        if isinstance(picMd5s, str):
+            picMd5s = [picMd5s]
+        picMd5s = [  # type: ignore
+            {"FileId": 1, "PicMd5": picmd5, "PicSize": 1} for picmd5 in picMd5s or []
+        ]
+        return self.post(
+            "SendMsgV2",
             {
-                "toUser": user,
-                "sendToType": 1,
-                "sendMsgType": "PicMsg",
-                "content": content,
-                "picUrl": picUrl,
-                "picBase64Buf": picBase64Buf,
-                "fileMd5": fileMd5,
-                "flashPic": flashPic,
+                "ToUserUid": user,
+                "SendToType": 1,
+                "SendMsgType": "PicMsg",
+                "Content": content,
+                "PicUrl": picUrl,
+                "PicBase64Buf": picBase64Buf,
+                "PicMd5s": picMd5s,
+                "FlashPic": flashPic,
             },
         )
 
@@ -130,33 +135,58 @@ class AsyncAction:
 
     async def sendGroupPic(
         self,
-        user: int,
+        group: int,
         *,
         content: str = "",
         picUrl: str = "",
         picBase64Buf: str = "",
-        fileMd5: str = "",
+        picMd5s: Union[str, List[str]] = None,
         flashPic=False,
         atUser: Union[int, List[int]] = 0,
     ) -> dict:
         """发送群组图片消息"""
-        assert any([picUrl, picBase64Buf, fileMd5]), "缺少参数"
+        assert any([picUrl, picBase64Buf, picMd5s]), "缺少参数"
         if atUser != 0:
             content = macro.atUser(atUser) + content
+        if isinstance(picMd5s, str):
+            picMd5s = [picMd5s]
+        picMd5s = [  # type: ignore
+            {"FileId": 1, "PicMd5": picmd5, "PicSize": 1} for picmd5 in picMd5s or []
+        ]
         return await self.post(
-            "SendMsg",
+            "SendMsgV2",
             {
-                "toUser": user,
-                "sendToType": 2,
-                "sendMsgType": "PicMsg",
-                "content": content,
-                "picUrl": picUrl,
-                "picBase64Buf": picBase64Buf,
-                "fileMd5": fileMd5,
-                "flashPic": flashPic,
+                "ToUserUid": group,
+                "SendToType": 2,
+                "SendMsgType": "PicMsg",
+                "Content": content,
+                "PicUrl": picUrl,
+                "PicBase64Buf": picBase64Buf,
+                "PicMd5s": picMd5s,
+                "FlashPic": flashPic,
             },
         )
 
+    async def sendGroupMultiPic(
+        self,
+        group,
+        *items: str,
+        text: str = "",
+        atUser: Union[int, List[int]] = 0,
+    ):
+        """发送群多图
+        items 支持填写图片http地址和base64，会自动判断类型
+        """
+        md5s = []
+        for item in items:
+            if item.startswith("http"):
+                info = await self.getGroupPicInfo(url=item)
+            else:
+                info = await self.getGroupPicInfo(base64=item)
+            md5s.append(info["PicInfo"]["PicMd5"])
+            time.sleep(0.5)
+        return await self.sendGroupPic(group, content=text, atUser=atUser, picMd5s=md5s)
+
     async def sendGroupVoice(
         self, group: int, *, voiceUrl: str = "", voiceBase64Buf: str = ""
     ) -> dict:
@@ -248,21 +278,26 @@ class AsyncAction:
         *,
         picUrl: str = "",
         picBase64Buf: str = "",
-        fileMd5: str = "",
+        picMd5s: Union[str, List[str]] = None,
     ) -> dict:
         """发送私聊图片消息"""
-        assert any([picUrl, picBase64Buf, fileMd5]), "缺少参数"
+        assert any([picUrl, picBase64Buf, picMd5s]), "缺少参数"
+        if isinstance(picMd5s, str):
+            picMd5s = [picMd5s]
+        picMd5s = [  # type: ignore
+            {"FileId": 1, "PicMd5": picmd5, "PicSize": 1} for picmd5 in picMd5s or []
+        ]
         return await self.post(
-            "SendMsg",
+            "SendMsgV2",
             {
-                "toUser": user,
-                "sendToType": 3,
-                "sendMsgType": "PicMsg",
-                "content": content,
-                "groupid": group,
-                "picUrl": picUrl,
-                "picBase64Buf": picBase64Buf,
-                "fileMd5": fileMd5,
+                "ToUserUid": user,
+                "GroupID": group,
+                "SendToType": 3,
+                "SendMsgType": "PicMsg",
+                "Content": content,
+                "PicUrl": picUrl,
+                "PicBase64Buf": picBase64Buf,
+                "PicMd5s": picMd5s,
             },
         )
 
@@ -513,20 +548,20 @@ class AsyncAction:
             {"GroupID": group, "UserID": user, "Flag": 0 if flag == 0 else 1},
         )
 
-    async def revokeGroupMsg(
-        self, group: int, msgSeq: int, msgRandom: int, cmd: int = 0
-    ) -> dict:
+    async def revokeGroupMsg(self, group: int, msgSeq: int, msgRandom: int) -> dict:
         """撤回群消息
         :param group: 群号
         :param msgSeq: 消息msgSeq
         :param msgRandom: 消息msgRandom
-        :param cmd: 0 或 1, 0表示使用: RevokeMsg , 1表示使用: PbMessageSvc.PbMsgWithDraw
         """
         return await self.post(
-            "RevokeMsg" if cmd == 0 else "PbMessageSvc.PbMsgWithDraw",
-            {"GroupID": group, "MsgSeq": msgSeq, "MsgRandom": msgRandom},
+            "RevokeMsg", {"GroupID": group, "MsgSeq": msgSeq, "MsgRandom": msgRandom}
         )
 
+    async def revoke(self, ctx: GroupMsg):
+        """撤回群消息"""
+        return await self.revokeGroupMsg(ctx.FromGroupId, ctx.MsgSeq, ctx.MsgRandom)
+
     async def joinGroup(self, group: int, content: str = "") -> dict:
         """加入群聊
         :param group: 哪个群?
@@ -556,6 +591,26 @@ class AsyncAction:
             {"ActionType": 3, "GroupID": group, "ActionUserID": user, "Content": ""},
         )
 
+    async def refreshKeys(self) -> dict:
+        """刷新key二次登陆"""
+        return await self.get("", path="/v1/RefreshKeys")
+
+    async def logout(self, flag=False) -> dict:
+        """退出指定QQ
+        :param flag: 是否删除设备信息文件
+        """
+        return await self.post("LogOut", {"Flag": flag})
+
+    async def getGroupPicInfo(self, url: str = "", base64: str = ""):
+        """上传群图片获取图片信息
+        :param url: 图片链接
+        :param base64: 图片base64
+        """
+        assert any([url, base64]), "缺少参数"
+        return await self.post(
+            "", {"PicUrl": url, "PicBase64Buf": base64}, path="/v1/GetGroupPicInfo"
+        )
+
     async def baseRequest(
         self,
         method: str,
